/*
 * CamView.java
 *
 * Created on April 5, 2006, 9:11 PM
 */

package com.penfieldrobotics.camview;

import javax.swing.*;
import java.awt.*;
import java.awt.event.*;

/**
 *
 */
public class CamView extends JPanel {
    private static final String NOT_LINKED_TEXT = "No";
    private static final String LINKED_TEXT = "Yes";
    private static final String NO_VALUE_TEXT = "---";
    public static final int PAN_CENTER_TOLERANCE = 2;
    private PanIndicator pangui;
    private TiltIndicator tiltgui;
    private CamData camdata;
    
    private JTextField linkedField;
    private JTextField panField;
    private JTextField tiltField;

    private JLabel leftArrow;
    private JLabel leftMoreArrow;
    private JLabel leftMostArrow;
    private JLabel rightArrow;
    private JLabel rightMoreArrow;
    private JLabel rightMostArrow;
    
    
    public CamView() {
        initGUI();
        newCamData(null);
    }
    
    public void newCamData(CamData camdata) {
        this.camdata = camdata;
        
        // Disable all the arrows. They are re-enabled below, as appropriate 
        leftMoreArrow.setEnabled(false);
        leftMostArrow.setEnabled(false);
        leftArrow.setEnabled(false);
        rightMoreArrow.setEnabled(false);
        rightMostArrow.setEnabled(false);
        rightArrow.setEnabled(false);
        
        if (camdata == null || !camdata.camsees) {
            pangui.setPan(-1);
            tiltgui.setTilt(-1);
            if (camdata == null)
                linkedField.setText(NOT_LINKED_TEXT);
            else
                linkedField.setText(LINKED_TEXT);
            panField.setText(NO_VALUE_TEXT);
            tiltField.setText(NO_VALUE_TEXT);
        } else {
            pangui.setPan(camdata.pan);
            tiltgui.setTilt(camdata.tilt);
            linkedField.setText(LINKED_TEXT);
            panField.setText(String.valueOf(camdata.pan));
            tiltField.setText(String.valueOf(camdata.tilt));

            // Enable the arrows according to how far off pan is from center.
            // 43 approximates 1/6th of a 256-value range.
            switch (camdata.pan / 43) {
                case 0:
                    leftMostArrow.setEnabled(true);
                    // Falls
                case 1:
                    leftMoreArrow.setEnabled(true);
                    // Falls
                case 2:
                    if (camdata.pan > 127 - PAN_CENTER_TOLERANCE)
                        break;
                    leftArrow.setEnabled(true);
                    break;
                case 5:
                    rightMostArrow.setEnabled(true);
                    // Falls
                case 4:
                    rightMoreArrow.setEnabled(true);
                    // Falls
                case 3:
                    if (camdata.pan < 127 + PAN_CENTER_TOLERANCE)
                        break;
                    rightArrow.setEnabled(true);
                    break;
            }
                    
                
        }
        repaint();
    }
    
    private void initGUI() {
        setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));
        
        pangui = new PanIndicator();
        pangui.setBackground(Color.BLACK);
        pangui.setForeground(Color.WHITE);
/*        pangui.setBorder(BorderFactory.createCompoundBorder(
            BorderFactory.createEmptyBorder(5, 5, 5, 5),
            BorderFactory.createBevelBorder(javax.swing.border.BevelBorder.RAISED)));
        add(pangui, BorderLayout.SOUTH);
*/
        tiltgui = new TiltIndicator();
        tiltgui.setBackground(Color.BLACK);
        tiltgui.setForeground(Color.RED);
        tiltgui.setOutlineColor(Color.BLUE);
        JPanel tiltPanel = new JPanel();
        tiltPanel.add(tiltgui);
/*        tiltPanel.setBorder(BorderFactory.createCompoundBorder(
            BorderFactory.createEmptyBorder(5, 5, 5, 5),
            BorderFactory.createBevelBorder(javax.swing.border.BevelBorder.RAISED)));
        add(tiltPanel, BorderLayout.CENTER);
  */    
        
        leftMostArrow = new JLabel(loadImageIcon("com/penfieldrobotics/camview/images/arrow-left-red-small.png"));
        leftMoreArrow = new JLabel(loadImageIcon("com/penfieldrobotics/camview/images/arrow-left-drkorange-small.png"));
        leftArrow = new JLabel(loadImageIcon("com/penfieldrobotics/camview/images/arrow-left-orange-small.png"));
        JPanel leftArrows = new JPanel();
        leftArrows.setLayout(new BoxLayout(leftArrows, BoxLayout.X_AXIS));
        leftArrows.add(Box.createRigidArea(new Dimension(10, 1)));
        leftArrows.add(leftMostArrow);
        leftArrows.add(Box.createRigidArea(new Dimension(5, 1)));
        leftArrows.add(leftMoreArrow);
        leftArrows.add(Box.createRigidArea(new Dimension(5, 1)));
        leftArrows.add(leftArrow);
        leftArrows.add(Box.createRigidArea(new Dimension(90, 1)));
        leftArrows.add(Box.createHorizontalGlue());

        
        JLabel logo = new JLabel(loadImageIcon("com/penfieldrobotics/camview/images/rollingthunder.jpg"));
        logo.setBorder(BorderFactory.createCompoundBorder(
            BorderFactory.createEmptyBorder(5, 5, 5, 5),
            BorderFactory.createBevelBorder(javax.swing.border.BevelBorder.LOWERED)));

        JPanel leftSide = new JPanel();
        leftSide.setLayout(new BoxLayout(leftSide, BoxLayout.Y_AXIS));
        logo.setAlignmentX(JComponent.CENTER_ALIGNMENT);
        leftArrows.setAlignmentX(JComponent.CENTER_ALIGNMENT);
        leftSide.add(Box.createRigidArea(new Dimension(1, 10)));
        leftSide.add(logo);
        leftSide.add(Box.createVerticalGlue());
        leftSide.add(leftArrows);


        rightMostArrow = new JLabel(loadImageIcon("com/penfieldrobotics/camview/images/arrow-right-red-small.png"));
        rightMoreArrow = new JLabel(loadImageIcon("com/penfieldrobotics/camview/images/arrow-right-drkorange-small.png"));
        rightArrow = new JLabel(loadImageIcon("com/penfieldrobotics/camview/images/arrow-right-orange-small.png"));
        JPanel rightArrows = new JPanel();
        rightArrows.setLayout(new BoxLayout(rightArrows, BoxLayout.X_AXIS));
        rightArrows.add(Box.createHorizontalGlue());
        rightArrows.add(Box.createRigidArea(new Dimension(90, 1)));
        rightArrows.add(rightArrow);
        rightArrows.add(Box.createRigidArea(new Dimension(5, 1)));
        rightArrows.add(rightMoreArrow);
        rightArrows.add(Box.createRigidArea(new Dimension(5, 1)));
        rightArrows.add(rightMostArrow);
        rightArrows.add(Box.createRigidArea(new Dimension(10, 1)));
        
        
        JLabel logo2 = new JLabel(loadImageIcon("com/penfieldrobotics/camview/images/firstlogo.png"));
        logo2.setBorder(BorderFactory.createCompoundBorder(
            BorderFactory.createEmptyBorder(5, 5, 5, 5),
            BorderFactory.createEmptyBorder(2, 2, 2, 2)));
            //BorderFactory.createEtchedBorder()));

        JPanel rightSide = new JPanel();
        rightSide.setLayout(new BoxLayout(rightSide, BoxLayout.Y_AXIS));
        logo2.setAlignmentX(JComponent.CENTER_ALIGNMENT);
        rightArrows.setAlignmentX(JComponent.CENTER_ALIGNMENT);
        rightSide.add(Box.createRigidArea(new Dimension(1, 10)));
        rightSide.add(logo2);
        rightSide.add(Box.createVerticalGlue());
        rightSide.add(rightArrows);
        
        JPanel topPanel = new JPanel();
        topPanel.setLayout(new BoxLayout(topPanel, BoxLayout.X_AXIS));
        topPanel.add(leftSide);
        topPanel.add(Box.createHorizontalStrut(10));
        topPanel.add(tiltgui);
        topPanel.add(Box.createHorizontalStrut(10));
        topPanel.add(rightSide);
        
        
        JPanel bottomPanel = new JPanel();
        bottomPanel.setLayout(new BoxLayout(bottomPanel, BoxLayout.X_AXIS));
        bottomPanel.add(pangui);
        

        JPanel linkPanel = new JPanel();
        linkPanel.add(new JLabel("Connected: "));
        linkedField = new JTextField(NOT_LINKED_TEXT, 5);
        linkedField.setEnabled(false);
        linkedField.setDisabledTextColor(Color.BLACK);
        //linkedField.setBorder(BorderFactory.createCompoundBorder(
        //            BorderFactory.createBevelBorder(javax.swing.border.BevelBorder.LOWERED),
        //            BorderFactory.createEmptyBorder(1,1,1,1)));
                    
        
        linkPanel.add(linkedField);
        
        panField = new JTextField(NO_VALUE_TEXT, 5);
        tiltField = new JTextField(NO_VALUE_TEXT, 5);
        panField.setEnabled(false);
        panField.setDisabledTextColor(Color.BLACK);
        tiltField.setEnabled(false);
        tiltField.setDisabledTextColor(Color.BLACK);
        JPanel ptPanel = new JPanel();
        ptPanel.add(new JLabel("Tilt: "));
        ptPanel.add(tiltField);
        ptPanel.add(new JLabel("Pan: "));
        ptPanel.add(panField);

        
        JPanel statusPanel = new JPanel();
        statusPanel.setLayout(new BoxLayout(statusPanel, BoxLayout.X_AXIS));
        statusPanel.add(linkPanel);
        statusPanel.add(Box.createRigidArea(new Dimension(5, 1)));
        statusPanel.add(ptPanel);
        statusPanel.add(Box.createHorizontalGlue());
        statusPanel.setMaximumSize(new Dimension(statusPanel.getMaximumSize().width, 
                                   statusPanel.getPreferredSize().height));
        statusPanel.setMinimumSize(statusPanel.getPreferredSize());
        
        add(Box.createVerticalStrut(10));
        add(topPanel);
        add(Box.createVerticalStrut(10));
        add(bottomPanel);
        add(Box.createVerticalStrut(10));
        add(statusPanel);
        setBorder(BorderFactory.createEmptyBorder(5, 5, 5, 5));
    }

    private static ImageIcon loadImageIcon(String path) {
        ClassLoader cl = CamView.class.getClassLoader();
        java.net.URL url = cl.getResource(path);
        return new ImageIcon(url);
    }
    
    public static void main(String[] args) throws Exception {
        CamView cv = new CamView();
        
        JFrame jf = new JFrame("Rolling Thunder CamView");
        jf.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
        jf.setIconImage(
            Toolkit.getDefaultToolkit().createImage(
                CamView.class.getClassLoader().getResource(
                    "com/penfieldrobotics/camview/images/aimhigh.png"
                )
            )
        );
        jf.getContentPane().add(cv);
        jf.pack();
//        jf.setSize(500, 500);
        jf.show();
        
        if (true) {
            Comm comm = new Comm();
            comm.addListener(cv);        
            comm.process();
        } else {
            FakeComm comm = new FakeComm(cv);
            comm.process();
        }
    }
    
}
