#ifndef IOMAP_H
#define IOMAP_H

#include "ifi_default.h"
#include "ifi_aliases.h"

/*
 * This file maps all the IFI I/O port names
 * to Team 1511 robot input/output functional
 * names. This makes it easy to handle pin reallocations
 */

/*
 * PWM Aliases.
 * 5, 6, 9 - 15 are currently unused
 * 16 is dedicated to providing power for the camera
 */

#define pwm_shooter_left_drive pwm03
#define pwm_shooter_right_drive pwm04
#define pwm_shooter_convey pwm05
#define pwm_collector_convey pwm06
#define pwm_camera_pan pwm07
#define pwm_camera_tilt pwm08
#define pwm_left_drive pwm11
#define pwm_right_drive pwm12


/*
 * Relay Aliases
 * None for now....
 * 1 - 8 Unused
 */


/* 
 * Analog inputs
 * 2 - 10, 16 currently unused
 */
#define anain_shooter_tilt rc_ana_in01

#define anain_autofast rc_ana_in07
#define anain_autodelay rc_ana_in08
#define anain_box_sw_right rc_ana_in09
#define anain_debug rc_ana_in10
#define anain_position_far rc_ana_in11
#define anain_position_near rc_ana_in12
#define anain_autoselect_msb rc_ana_in13
#define anain_autoselect_midb rc_ana_in14
#define anain_autoselect_lsb rc_ana_in15
#define anain_box_pot rc_ana_in16


/* Digital I/O
 * NOTE: If you change the direction of the pin, you
 *       must change the init code in 
 *       user_routines.c::User_Initialization()
 * 1, 2, 9-18 are not in use
 */

// Encoder driver needs this set of definitions to be consistent
#define digin_left_wheel_enc rc_dig_in03
#define ENABLE_ENCODER_3
#define Get_Left_Encoder_Count Get_Encoder_3_Count
#define Reset_Left_Encoder_Count Reset_Encoder_3_Count

#define digin_right_wheel_enc rc_dig_in04
#define ENABLE_ENCODER_4
#define Get_Right_Encoder_Count Get_Encoder_4_Count
#define Reset_Right_Encoder_Count Reset_Encoder_4_Count

#define digin_shooter_left_enc rc_dig_in05
#define ENABLE_ENCODER_5
#define Get_Left_Shooter_Encoder_Count Get_Encoder_5_Count
#define Reset_Left_Shooter_Encoder_Count Reset_Encoder_5_Count

#define digin_shooter_right_enc rc_dig_in06
#define ENABLE_ENCODER_6
#define Get_Right_Shooter_Encoder_Count Get_Encoder_6_Count
#define Reset_Right_Shooter_Encoder_Count Reset_Encoder_6_Count

#define digin_banner_sensor rc_dig_in07
#define digin_speed_banner rc_dig_in08

#define digout_box_led_left rc_dig_out16
#define digout_box_led_mid rc_dig_out17
#define digout_box_led_right rc_dig_out18

// OI Input map
#define oi_right_drive p2_y
#define oi_turbo p2_sw_trig
#define oi_left_drive p4_y
#define oi_turbo2 p4_sw_trig
#define oi_collector_fwd p2_sw_aux1
#define oi_collector_rev p2_sw_aux2

#define oi_feed_fwd p1_sw_aux1
#define oi_feed_rev p1_sw_aux2
#define oi_launch p3_sw_trig
#define oi_autoaim_off p3_sw_top
#define oi_aim p3_sw_aux1
#define oi_launch_speed p3_wheel
#define oi_launch_nkill p3_x

// OI LEDs
#define oi_led_feed_fwd Pwm1_green
#define oi_led_feed_rev Pwm1_red
#define oi_led_mode_normal Pwm2_green
#define oi_led_mode_debug Pwm2_red
#define oi_led_camera_locked Relay1_green
#define oi_led_ball_ready Relay1_red
#define oi_led_aim_locked Relay2_green
#define oi_led_autoaim_off Relay2_red




#endif
