/*******************************************************************************
* FILE NAME: user_routines.c <FRC VERSION>
*
* DESCRIPTION:
*  This file contains the default mappings of inputs  
*  (like switches, joysticks, and buttons) to outputs on the RC.  
*
* USAGE:
*  You can either modify this file to fit your needs, or remove it from your 
*  project and replace it with a modified copy. 
*
*******************************************************************************/

#include <stdio.h>

#include "ifi_aliases.h"
#include "ifi_default.h"
#include "ifi_utilities.h"
#include "user_routines.h"
#include "serial_ports.h"
#include "camera.h"
#include "tracking.h"
#include "encoder.h"
#include "trajectory.h"

// Drive code LUTs
static const rom unsigned char drive_lut[];
static const rom unsigned char turbo_lut[];

// Function prototypes
static void Manual_Drive(void);
static void Auto_Aim(void);


// Global defs
int normal_mode = 1;

static int is_aimed = 0;


/*** DEFINE USER VARIABLES AND INITIALIZE THEM HERE ***/
/* EXAMPLES: (see MPLAB C18 User's Guide, p.9 for all types)
unsigned char wheel_revolutions = 0; (can vary from 0 to 255)
unsigned int  delay_count = 7;       (can vary from 0 to 65,535)
int           angle_deviation = 142; (can vary from -32,768 to 32,767)
unsigned long very_big_counter = 0;  (can vary from 0 to 4,294,967,295)
*/

/*******************************************************************************
* FUNCTION NAME: User_Initialization
* PURPOSE:       This routine is called first (and only once) in the Main function.  
*                You may modify and add to this function.
* CALLED FROM:   main.c
* ARGUMENTS:     none
* RETURNS:       void
*******************************************************************************/
void User_Initialization (void)
{
  Set_Number_of_Analog_Channels(SIXTEEN_ANALOG);    /* DO NOT CHANGE! */

  digital_io_01 = digital_io_02 = digital_io_03 = digital_io_04 = INPUT;
  digital_io_05 = digital_io_06 = digital_io_07 = digital_io_08 = INPUT;
  digital_io_09 = digital_io_10 = digital_io_11 = digital_io_12 = INPUT;
  digital_io_13 = digital_io_14 = digital_io_15 = INPUT;
  digital_io_16 = OUTPUT;
  digital_io_17 = OUTPUT;
  digital_io_18 = OUTPUT;    /* Example - Not used in Default Code. */

  rc_dig_out16 = 0;
  rc_dig_out17 = 0;
  rc_dig_out18 = 0;

/* FOURTH: Set your initial PWM values.  Neutral is 127. */
  pwm01 = pwm02 = pwm03 = pwm04 = pwm05 = pwm06 = pwm07 = pwm08 = 127;
  pwm09 = pwm10 = pwm11 = pwm12 = pwm13 = pwm14 = pwm15 = pwm16 = 127;

/* FIFTH: Set your PWM output types for PWM OUTPUTS 13-16.
  /*   Choose from these parameters for PWM 13-16 respectively:               */
  /*     IFI_PWM  - Standard IFI PWM output generated with Generate_Pwms(...) */
  /*     USER_CCP - User can use PWM pin as digital I/O or CCP pin.           */
  Setup_PWM_Output_Type(IFI_PWM,IFI_PWM,IFI_PWM,IFI_PWM);

  /* 
     Example: The following would generate a 40KHz PWM with a 50% duty cycle on the CCP2 pin:

         CCP2CON = 0x3C;
         PR2 = 0xF9;
         CCPR2L = 0x7F;
         T2CON = 0;
         T2CONbits.TMR2ON = 1;

         Setup_PWM_Output_Type(USER_CCP,IFI_PWM,IFI_PWM,IFI_PWM);
  */

  /* Add any other initialization code here. */
 
#if defined(ENABLE_SERIAL_PORT_ONE_RX) || defined(ENABLE_SERIAL_PORT_ONE_TX)
  Init_Serial_Port_One();
  stdout_serial_port = SERIAL_PORT_ONE;
#endif
#if defined(ENABLE_SERIAL_PORT_ONE_TX) || defined(ENABLE_SERIAL_PORT_TWO_TX)
  Init_Serial_Port_Two();
#endif

  Initialize_Encoders();
  Initialize_Camera();

  Initialize_Timer_1();




  printf("IFI 2006 User Processor Initialized ...\r");  /* Optional - Print initialization message. */

  Putdata(&txdata);             /* DO NOT CHANGE! */
  
  User_Proc_Is_Ready();         /* DO NOT CHANGE! - last line of User_Initialization */

  // Set the bar to get to debug very high to prevent 
  // a floating input from putting us into debug when, say,
  // the box inadvertantly becomes unplugged....
  normal_mode = (Get_Analog_Value(anain_debug) > 700) ? 0 : 1;

  printf("Normal Mode: %d (Raw = %d)\r\n", normal_mode, Get_Analog_Value(anain_debug));

  printf("Remember.. these are active low!\r\n");
  printf("Box sw Autofast: %d Delay: %d Right: %d\r\n", Get_Analog_Value(anain_autofast), Get_Analog_Value(anain_autodelay), Get_Analog_Value(anain_box_sw_right));
  printf("Box pos far: %d near: %d\r\n", Get_Analog_Value(anain_position_far), Get_Analog_Value(anain_position_near));

  printf("Box POT: %d\r\n", Get_Analog_Value(anain_box_pot));
  
  digout_box_led_left = Get_Analog_Value(anain_autoselect_msb) > 500;
  digout_box_led_mid = Get_Analog_Value(anain_autoselect_midb) > 500;
  digout_box_led_right = Get_Analog_Value(anain_autoselect_lsb) > 500;


}

/*******************************************************************************
* FUNCTION NAME: Process_Data_From_Master_uP
* PURPOSE:       Executes every 26.2ms when it gets new data from the master 
*                microprocessor.
* CALLED FROM:   main.c
* ARGUMENTS:     none
* RETURNS:       void
*******************************************************************************/
void Process_Data_From_Master_uP(void)
{
  static unsigned char i;

  Getdata(&rxdata);   /* Get fresh data from the master microprocessor. */

  Camera_Handler();
	Servo_Track();

  Default_Routine();  /* Optional.  See below. */

#if 0
pwm_left_drive = 127;
pwm_right_drive = 127;
PAN_SERVO = 127;//oi_right_drive;
TILT_SERVO = 127;//oi_left_drive;
#endif

  //printf("p1y: %d p1x: %d trig: %d top: %d aux: %d aux2: %d\r\n",p1_y,p1_x,p1_sw_trig,p1_sw_top,p1_sw_aux1,p1_sw_aux2);

  /* Add your own code here. (a printf will not be displayed when connected to 
  the breaker panel unless a Y cable is used) */
  
  //Generate_Pwms(pwm13,pwm14,pwm15,pwm16);
  Putdata(&txdata);             /* DO NOT CHANGE! */
}

/*******************************************************************************
* FUNCTION NAME: Default_Routine
* PURPOSE:       Performs the default mappings of inputs to outputs for the
*                Robot Controller.
* CALLED FROM:   this file, Process_Data_From_Master_uP routine
* ARGUMENTS:     none
* RETURNS:       void
*******************************************************************************/
void Default_Routine(void)
{
  
	/*---------- Analog Inputs (Joysticks) to PWM Outputs-----------------------
	 *--------------------------------------------------------------------------
	 *   This maps the joystick axes to specific PWM outputs.
	 */
  Manual_Drive();

// don't reset shooter-changed for RPS control
#ifndef RPS_SHOOTER_CONTROL
//  pwm_shooter_left_drive = pwm_shooter_right_drive = 127;
#endif


  if (!oi_autoaim_off) {
    Auto_Aim();
  }


  // Kill launcher if OI pot is turned all the way off
  // For emergency use in case conveyer breaks.
  if (oi_launch_nkill < 250)
    Launch_Handler();
  else {
    pwm_shooter_convey = 127;
    Launcher_Spindown();
  }

  // Handle the lower ball collector input from the box
	if(oi_collector_fwd)
		pwm_collector_convey = 255;
	else if(oi_collector_rev)
		pwm_collector_convey = 0;
	else
		pwm_collector_convey = 127;

  // Update OI LEDs and user byte feedback
  FeedbackUpdate();
} 


static void Manual_Drive(void)
{
	if(!oi_turbo && !oi_turbo2)
	{
		pwm_left_drive = 255 - drive_lut[oi_left_drive];
		pwm_right_drive = 255 - drive_lut[oi_right_drive];
	}
	else
	{
		pwm_left_drive = 255 - turbo_lut[oi_left_drive];
		pwm_right_drive = 255 - (int)turbo_lut[oi_right_drive];
	}  
}

//this strictly aims the robot according to the camera
static void Auto_Aim(void)
{
  int is_oriented = 0;
	if(oi_aim) {
    // if the aim button is pushed
		if(!camera_locked) {
			pwm_left_drive = 150;
			pwm_right_drive = 104;
		}	else {
			is_oriented = (char) Robot_Orient();
    }
	}

  is_aimed = is_oriented;
}  





// Updates OI LEDs
void FeedbackUpdate(void)
{
  if (user_display_mode == 0) {
    // User Mode is Off

    if (normal_mode) {
      oi_led_mode_normal = 1;
      oi_led_mode_debug = 0;
      oi_led_feed_fwd = (oi_feed_fwd == 1);
      oi_led_feed_rev = (oi_feed_rev == 1);
      oi_led_autoaim_off = oi_autoaim_off;
      oi_led_camera_locked = camera_locked;

      if (oi_aim) {
        if (is_aimed) {
          oi_led_aim_locked = 1;
        } else {
          static int aim_blink_count = 0;
          if (++aim_blink_count > 8) {
            oi_led_aim_locked = !oi_led_aim_locked;
            aim_blink_count = 0;
          }
        }
      } else {
        oi_led_aim_locked = 0;
      }

    } else {
      oi_led_mode_debug = 1;
      oi_led_mode_normal = 0;

  		oi_led_feed_fwd = digin_shooter_left_enc;
	  	oi_led_feed_rev = digin_shooter_right_enc;
      oi_led_autoaim_off = digin_right_wheel_enc;
      oi_led_camera_locked = digin_left_wheel_enc;
    }

    // Banner sensor is negative logic, so invert
    // This is so basic, it applies to debug and normal mode.
    oi_led_ball_ready = !digin_banner_sensor;

  } else {
   /* User Mode is On - displays data in OI 4-digit display*/
    User_Mode_byte = backup_voltage*10; /* so that decimal doesn't get truncated. */
  }   
}



/* LUTs for manual drive */
static const rom unsigned char drive_lut[] = {
	0x37, 0x39, 0x3B, 0x3C, 0x3E, 0x40, 0x41, 0x43, 
	0x44, 0x46, 0x47, 0x49, 0x4A, 0x4B, 0x4D, 0x4E, 
	0x50, 0x51, 0x52, 0x53, 0x55, 0x56, 0x57, 0x58, 
	0x59, 0x5A, 0x5C, 0x5D, 0x5E, 0x5F, 0x60, 0x61, 
	0x62, 0x63, 0x64, 0x64, 0x65, 0x66, 0x67, 0x68, 
	0x69, 0x6A, 0x6A, 0x6B, 0x6C, 0x6D, 0x6D, 0x6E, 
	0x6F, 0x6F, 0x70, 0x71, 0x71, 0x72, 0x72, 0x73, 
	0x73, 0x74, 0x74, 0x75, 0x75, 0x76, 0x76, 0x77, 
	0x77, 0x78, 0x78, 0x78, 0x79, 0x79, 0x79, 0x7A, 
	0x7A, 0x7A, 0x7B, 0x7B, 0x7B, 0x7C, 0x7C, 0x7C, 
	0x7C, 0x7C, 0x7D, 0x7D, 0x7D, 0x7D, 0x7D, 0x7E, 
	0x7E, 0x7E, 0x7E, 0x7E, 0x7E, 0x7E, 0x7F, 0x7F, 
	0x7F, 0x7F, 0x7F, 0x7F, 0x7F, 0x7F, 0x7F, 0x7F, 
	0x7F, 0x7F, 0x7F, 0x7F, 0x7F, 0x7F, 0x7F, 0x7F, 
	0x7F, 0x7F, 0x7F, 0x7F, 0x7F, 0x7F, 0x7F, 0x7F, 
	0x7F, 0x7F, 0x7F, 0x7F, 0x7F, 0x7F, 0x7F, 0x7F, 
	0x7F, 0x7F, 0x7F, 0x7F, 0x7F, 0x7F, 0x7F, 0x7F, 
	0x7F, 0x7F, 0x7F, 0x7F, 0x7F, 0x7F, 0x7F, 0x7F, 
	0x7F, 0x7F, 0x7F, 0x7F, 0x7F, 0x7F, 0x7F, 0x7F, 
	0x7F, 0x7F, 0x7F, 0x7F, 0x7F, 0x7F, 0x7F, 0x7F, 
	0x7F, 0x80, 0x80, 0x80, 0x80, 0x80, 0x80, 0x80, 
	0x81, 0x81, 0x81, 0x81, 0x81, 0x82, 0x82, 0x82, 
	0x82, 0x82, 0x83, 0x83, 0x83, 0x84, 0x84, 0x84, 
	0x85, 0x85, 0x85, 0x86, 0x86, 0x86, 0x87, 0x87, 
	0x88, 0x88, 0x89, 0x89, 0x8A, 0x8A, 0x8B, 0x8B, 
	0x8C, 0x8C, 0x8D, 0x8D, 0x8E, 0x8F, 0x8F, 0x90, 
	0x91, 0x91, 0x92, 0x93, 0x94, 0x94, 0x95, 0x96, 
	0x97, 0x98, 0x99, 0x9A, 0x9A, 0x9B, 0x9C, 0x9D, 
	0x9E, 0x9F, 0xA0, 0xA1, 0xA2, 0xA4, 0xA5, 0xA6, 
	0xA7, 0xA8, 0xA9, 0xAB, 0xAC, 0xAD, 0xAE, 0xB0, 
	0xB1, 0xB3, 0xB4, 0xB5, 0xB7, 0xB8, 0xBA, 0xBB, 
	0xBD, 0xBE, 0xC0, 0xC2, 0xC3, 0xC5, 0xC7, 0xC8
};

static const rom unsigned char turbo_lut[] = {
	0x00, 0x03, 0x06, 0x09, 0x0C, 0x0E, 0x11, 0x14, 
	0x17, 0x19, 0x1C, 0x1E, 0x21, 0x23, 0x26, 0x28, 
	0x2A, 0x2D, 0x2F, 0x31, 0x33, 0x35, 0x37, 0x3A, 
	0x3C, 0x3E, 0x3F, 0x41, 0x43, 0x45, 0x47, 0x49, 
	0x4A, 0x4C, 0x4E, 0x4F, 0x51, 0x52, 0x54, 0x55, 
	0x57, 0x58, 0x59, 0x5B, 0x5C, 0x5D, 0x5E, 0x60, 
	0x61, 0x62, 0x63, 0x64, 0x65, 0x66, 0x67, 0x68, 
	0x69, 0x6A, 0x6B, 0x6C, 0x6D, 0x6E, 0x6E, 0x6F, 
	0x70, 0x71, 0x71, 0x72, 0x73, 0x73, 0x74, 0x74, 
	0x75, 0x76, 0x76, 0x77, 0x77, 0x78, 0x78, 0x78, 
	0x79, 0x79, 0x7A, 0x7A, 0x7A, 0x7B, 0x7B, 0x7B, 
	0x7B, 0x7C, 0x7C, 0x7C, 0x7C, 0x7D, 0x7D, 0x7D, 
	0x7D, 0x7D, 0x7D, 0x7E, 0x7E, 0x7E, 0x7E, 0x7E, 
	0x7E, 0x7E, 0x7E, 0x7E, 0x7E, 0x7E, 0x7F, 0x7F, 
	0x7F, 0x7F, 0x7F, 0x7F, 0x7F, 0x7F, 0x7F, 0x7F, 
	0x7F, 0x7F, 0x7F, 0x7F, 0x7F, 0x7F, 0x7F, 0x7F, 
	0x7E, 0x7E, 0x7E, 0x7E, 0x7E, 0x7E, 0x7E, 0x7E, 
	0x7E, 0x7E, 0x7E, 0x7E, 0x7E, 0x7E, 0x7E, 0x7E, 
	0x7F, 0x7F, 0x7F, 0x7F, 0x7F, 0x7F, 0x7F, 0x7F, 
	0x7F, 0x7F, 0x7F, 0x7F, 0x80, 0x80, 0x80, 0x80, 
	0x80, 0x80, 0x81, 0x81, 0x81, 0x81, 0x82, 0x82, 
	0x82, 0x82, 0x83, 0x83, 0x83, 0x84, 0x84, 0x85, 
	0x85, 0x85, 0x86, 0x86, 0x87, 0x87, 0x88, 0x89, 
	0x89, 0x8A, 0x8A, 0x8B, 0x8C, 0x8C, 0x8D, 0x8E, 
	0x8F, 0x8F, 0x90, 0x91, 0x92, 0x93, 0x94, 0x95, 
	0x96, 0x97, 0x98, 0x99, 0x9A, 0x9B, 0x9C, 0x9D, 
	0x9F, 0xA0, 0xA1, 0xA2, 0xA4, 0xA5, 0xA6, 0xA8, 
	0xA9, 0xAB, 0xAC, 0xAE, 0xAF, 0xB1, 0xB3, 0xB4, 
	0xB6, 0xB8, 0xBA, 0xBC, 0xBE, 0xBF, 0xC1, 0xC3, 
	0xC6, 0xC8, 0xCA, 0xCC, 0xCE, 0xD0, 0xD3, 0xD5, 
	0xD7, 0xDA, 0xDC, 0xDF, 0xE1, 0xE4, 0xE6, 0xE9, 
	0xEC, 0xEF, 0xF1, 0xF4, 0xF7, 0xFA, 0xFD, 0xFF
};

/******************************************************************************/
/******************************************************************************/
/******************************************************************************/

