/*
 * Comm.java
 *
 * Created on April 4, 2006, 10:12 PM
 */

package com.penfieldrobotics.camview;

import gnu.io.*;
import java.util.*;
import java.io.*;

/**
 *
 */
public class Comm {
    // This is the byte offset for pwm8
    private static final int TILT_BYTE_OFFSET = 16;
    // This is the byte offset for pwm7
    private static final int PAN_BYTE_OFFSET = 14;
    // LED byte 1
    private static final int LED_BYTE_OFFSET = 19;
    // Relay1_green led
    private static final int CAM_LOCKED_BIT = 0x20;

    private SerialPort serialport;
    private InputStream input;
    private byte[] inputBuffer;
    private boolean inFrame;
    
    private CamView listener;
    
    public Comm() throws NoSuchPortException, PortInUseException, 
                         UnsupportedCommOperationException,
                         IOException
    {
        
        CommPortIdentifier cpi = CommPortIdentifier.getPortIdentifier("COM1");
        serialport = (SerialPort)cpi.open("Camera App", 1000);
        serialport.setSerialPortParams(19200, SerialPort.DATABITS_8, 
                                              SerialPort.STOPBITS_1,
                                              SerialPort.PARITY_NONE);
        input = serialport.getInputStream();
        inputBuffer = new byte[26*3];
    }
    
    public void addListener(CamView cv) {
        this.listener = cv;
    }

    public void process() throws IOException {
        int frameState = 0;
        inFrame = false;

        while (true) {
            if (inFrame) {
                inFrame = processFrame();
            } else {
                // Let listener know we lost synch
                fireDataEvent(null);
                inFrame = findFrame();
            }
        }
    }
    
    
    private boolean processFrame() throws IOException {
        // Process the frame at the head of the buffer
        if ((inputBuffer[0] & 0xFF) != 0xFF || (inputBuffer[1] & 0xFF) != 0xFF)
            return false;
        
        int ctrla = inputBuffer[7] & 0xFF;
        int ctrlc = inputBuffer[11] & 0xFF;
        
        if ((ctrlc & 0x80) == 0 && (ctrla & 0x10) == 0) {
            // Legacy data frame
System.out.println("Locked: " + Integer.toHexString(inputBuffer[LED_BYTE_OFFSET]));
            CamData cd = new CamData(inputBuffer[PAN_BYTE_OFFSET] & 0xFF, 
                                     inputBuffer[TILT_BYTE_OFFSET] & 0xFF,
                                     (inputBuffer[LED_BYTE_OFFSET] & CAM_LOCKED_BIT) != 0);
            fireDataEvent(cd);
        }
        
        // Copy down rest of buffer
        System.arraycopy(inputBuffer, 26, inputBuffer, 0, inputBuffer.length - 26);
        
        // Read in new packet at end of buffer
        int len = input.read(inputBuffer, inputBuffer.length - 26, 26);
        while (len < 26)
            len += input.read(inputBuffer, inputBuffer.length - 26 + len, 
                              26 - len);
        
        return true;
    }
    
    private void fireDataEvent(final CamData cd) {
        if (cd == null) {
            System.out.println("Lost frame....");
        } else {
            System.out.println("Pan: " + cd.pan);
            System.out.println("Tilt: " + cd.tilt + "\n");
        }
        Runnable r = new Runnable() {
            public void run() {
                listener.newCamData(cd);
            }
        };
        javax.swing.SwingUtilities.invokeLater(r);
    }
    
    private boolean findFrame() throws IOException {
        int len = input.read(inputBuffer, inputBuffer.length - 26, 26);
        while (len < 26) {
            len += input.read(inputBuffer, inputBuffer.length - 26 + len, 
                              26 - len);
        }
        
        len = inputBuffer.length;
        int frameStart = -1;
        for (int i = 0; i < 25; ++i) {
            if ((inputBuffer[i] & 0xFF) == 0xFF && (inputBuffer[i + 1] & 0xFF) == 0xFF &&
                (inputBuffer[i + 26] & 0xFF) == 0xFF && (inputBuffer[i + 27] & 0xFF)  == 0xFF &&
                (inputBuffer[i + 52] & 0xFF) == 0xFF && (inputBuffer[i + 53] & 0xFF) == 0xFF) {
                
                frameStart = i;
                len = (26 - i) + 26 * 2;
                
                break;
            }
        }
        
        if (frameStart == -1) {
            System.arraycopy(inputBuffer, 26,
                             inputBuffer, 0, 26 * 2);
            return false;
        }
        
        // Shift good 23 frames to beginning of buffer
        System.arraycopy(inputBuffer, frameStart, inputBuffer, 0, len);
        
        // Complete final packet
        while (len < inputBuffer.length)
            len += input.read(inputBuffer, len, inputBuffer.length - len);

        return true;
    }
    
    
    
    
    public static void main(String[] args) throws Exception {
        Enumeration e = CommPortIdentifier.getPortIdentifiers();
        while (e.hasMoreElements()) {
            CommPortIdentifier cpi = (CommPortIdentifier)e.nextElement();
            System.out.println(cpi.getName());
        }
        
        Comm c = new Comm();
        c.process();
    }
    
}
