#include "strategy.h"
#include "user_routines.h"
#include "encoder.h"
#include "trajectory.h"
#include "ifi_utilities.h"
#include "tracking.h"
#include <stdio.h>

// 196 
#define MAGNITUDE 177

#define CAMPAN_TOLERANCE 8
static const int CAMPAN_LOW_BOUND = CAMPAN_CENTER - CAMPAN_TOLERANCE;
static const int CAMPAN_HIGH_BOUND = CAMPAN_CENTER + CAMPAN_TOLERANCE;


static int state = 0;

void Reset_Autonomous(void)
{
  state = 0;
}


void Drive_And_Shoot(int distance, int magnitude, int useCam)
{
  static int wait = 0;
  int fire = 0;

  switch (state) {
    case 0:
        Reset_Left_Encoder_Count();
        Reset_Right_Encoder_Count();
        ++state;
        /* Fall through */
    case 1:
    	if(Get_Left_Encoder_Count() < distance && 
         Get_Right_Encoder_Count() < distance) {
     	  pwm_left_drive = pwm_right_drive = 255 - magnitude;
        break;
      } else {
    	  pwm_left_drive = pwm_right_drive = 127;
        ++state;
        wait = 5;
      }
      /* Fall */
    case 2:
      if (--wait != 0) {
        break;
      } else {
        Reset_Left_Encoder_Count();
        Reset_Right_Encoder_Count();        
        ++state;
      }
      /* Fall */
    case 3:
      if (useCam && 
              (!camera_locked || 
               PAN_SERVO < CAMPAN_LOW_BOUND || 
               PAN_SERVO > CAMPAN_HIGH_BOUND))
        fire = 0;
      else
        fire = 1;
  }

  Auto_Launch_Handler(fire);


}





void Straight_Defense(int magnitude, int goforever)
{
  switch (state) {
    case 0:
        Reset_Left_Encoder_Count();
        Reset_Right_Encoder_Count();
        ++state;
        /* Fall through */
    case 1:
    	if(goforever ||
         (Get_Left_Encoder_Count() < DISTANCE_CROSS_WHOLE && 
          Get_Right_Encoder_Count() < DISTANCE_CROSS_WHOLE)) {
    	  pwm_left_drive = pwm_right_drive = 255 - MAGNITUDE;
        break;
      } else {
        ++state;
      }
      // Fall
    case 2:
      pwm_left_drive = pwm_right_drive = 127;
      break;
	}
  
}

void Out_And_Turn(int d_out, int d_cross, int willshoot)
{
  static int wait = 0;
  int fire = 0;

  switch (state) {
    case 0:
        Reset_Left_Encoder_Count();
        Reset_Right_Encoder_Count();
        ++state;
        /* Fall through */
    case 1:
    	if(Get_Left_Encoder_Count() < d_out && 
         Get_Right_Encoder_Count() < d_out) {
    	  pwm_left_drive = pwm_right_drive = 255 - MAGNITUDE;
        break;
      } else {
    	  pwm_left_drive = pwm_right_drive = 127;
        ++state;
        wait = 5;
      }
      /* Fall */
    case 2:
      if (--wait != 0) {
        break;
      } else {
        Reset_Left_Encoder_Count();
        Reset_Right_Encoder_Count();        
        ++state;
      }
      /* Fall */
    case 3:
    	if(Get_Left_Encoder_Count() < DISTANCE_TURN_90 && 
         Get_Right_Encoder_Count() < DISTANCE_TURN_90) {
        // Turning right
    	  pwm_left_drive = 255 - MAGNITUDE;
        pwm_right_drive = MAGNITUDE;
        break;
      } else {
    	  pwm_left_drive = pwm_right_drive = 127;
        ++state;
        wait = 5;
      }
      /* Fall */
    case 4:
      if (--wait != 0) {
        break;
      } else {
        Reset_Left_Encoder_Count();
        Reset_Right_Encoder_Count();        
        ++state;
      }
      /* Fall */
    case 5:
    	if(Get_Left_Encoder_Count() < d_cross && 
         Get_Right_Encoder_Count() < d_cross) {
    	  pwm_left_drive = pwm_right_drive = 255 - MAGNITUDE;
        break;
      } else {
    	  pwm_left_drive = pwm_right_drive = 127;
        ++state;
        wait = 5;
      }
      /* Fall */
    case 6:
      if (--wait != 0) {
        break;
      } else {
        Reset_Left_Encoder_Count();
        Reset_Right_Encoder_Count();        
        ++state;
      }
      /* Fall */
    case 7:
//      if (!Robot_Orient())
  //      break;
      ++state;
      // Fall through...
    case 8:
      fire = 1;
	}

  if (willshoot)
    Auto_Launch_Handler(fire);
}




void Sideways_Defense(int d_out, int d_end)
{
  static int wait = 0;
  switch (state) {
    case 0:
        Reset_Left_Encoder_Count();
        Reset_Right_Encoder_Count();
        ++state;
        /* Fall through */
    case 1:
      // Drive out of starting pos
    	if(Get_Left_Encoder_Count() < d_out && 
         Get_Right_Encoder_Count() < d_out) {
    	  pwm_left_drive = pwm_right_drive = 255 - MAGNITUDE;
        break;
      } else {
    	  pwm_left_drive = pwm_right_drive = 127;
        ++state;
        wait = 3;
      }
      /* Fall */
    case 2:
      if (--wait != 0) {
        break;
      } else {
        Reset_Left_Encoder_Count();
        Reset_Right_Encoder_Count();        
        ++state;
      }
      /* Fall */
    case 3:
      // Turning right
    	if(Get_Left_Encoder_Count() < DISTANCE_TURN_90 && 
         Get_Right_Encoder_Count() < DISTANCE_TURN_90) {
    	  pwm_left_drive = 255 - MAGNITUDE;
        pwm_right_drive = MAGNITUDE;
        break;
      } else {
    	  pwm_left_drive = pwm_right_drive = 127;
        ++state;
        wait = 3;
      }
      /* Fall */
    case 4:
      if (--wait != 0) {
        break;
      } else {
        Reset_Left_Encoder_Count();
        Reset_Right_Encoder_Count();        
        ++state;
      }
      /* Fall */
    case 5:
      // Drive to the end of the starting positions
    	if(Get_Left_Encoder_Count() < d_end && 
         Get_Right_Encoder_Count() < d_end) {
    	  pwm_left_drive = pwm_right_drive = 255 - MAGNITUDE;
        break;
      } else {
    	  pwm_left_drive = pwm_right_drive = 127;
        ++state;
        wait = 3;
      }
      /* Fall */
    case 6:
      if (--wait != 0) {
        break;
      } else {
        Reset_Left_Encoder_Count();
        Reset_Right_Encoder_Count();        
        ++state;
      }
      /* Fall */
    case 7:
stepseven:
      // Drive in reverse to the other end of the starting
      // positions
    	if(Get_Left_Encoder_Count() < DISTANCE_CROSS_SPOS && 
         Get_Right_Encoder_Count() < DISTANCE_CROSS_SPOS) {
    	  pwm_left_drive = pwm_right_drive = MAGNITUDE;
        break;
      } else {
    	  pwm_left_drive = pwm_right_drive = 127;
        ++state;
        wait = 3;
      }
      /* Fall */
    case 8:
      if (--wait != 0) {
        break;
      } else {
        Reset_Left_Encoder_Count();
        Reset_Right_Encoder_Count();        
        ++state;
      }
      /* Fall */
    case 9:
      // Drive forward, back to the other end of the
      // starting positions
    	if(Get_Left_Encoder_Count() < DISTANCE_CROSS_SPOS && 
         Get_Right_Encoder_Count() < DISTANCE_CROSS_SPOS) {
    	  pwm_left_drive = pwm_right_drive = 255 - MAGNITUDE;
        break;
      } else {
    	  pwm_left_drive = pwm_right_drive = 127;
        ++state;
        wait = 3;
      }
      /* Fall */
    case 10:
      if (--wait != 0) {
        break;
      } else {
        Reset_Left_Encoder_Count();
        Reset_Right_Encoder_Count();        
        state = 7;
        goto stepseven;
      }
  }
}

void Cross_Defense(void)
{
  static int wait = 0;
  switch (state) {
    case 0:
        Reset_Left_Encoder_Count();
        Reset_Right_Encoder_Count();
        ++state;
        /* Fall through */
    case 1:
    	if(Get_Left_Encoder_Count() < DISTANCE_CROSS_2THIRD && 
         Get_Right_Encoder_Count() < DISTANCE_CROSS_2THIRD) {
    	  pwm_left_drive = pwm_right_drive = 255 - MAGNITUDE;
        break;
      } else {
    	  pwm_left_drive = pwm_right_drive = 127;
        ++state;
        wait = 3;
      }
      /* Fall */
    case 2:
      if (--wait != 0) {
        break;
      } else {
        Reset_Left_Encoder_Count();
        Reset_Right_Encoder_Count();        
        ++state;
      }
      /* Fall */
    case 3:
statethree:
    	if(Get_Left_Encoder_Count() < DISTANCE_CROSS_THIRD && 
         Get_Right_Encoder_Count() < DISTANCE_CROSS_THIRD) {
        // Go in Reverse
    	  pwm_right_drive = MAGNITUDE;
        pwm_left_drive = MAGNITUDE;
        break;
      } else {
    	  pwm_left_drive = pwm_right_drive = 127;
        ++state;
        wait = 3;
      }
      /* Fall */
    case 4:
      if (--wait != 0) {
        break;
      } else {
        Reset_Left_Encoder_Count();
        Reset_Right_Encoder_Count();
        ++state;
      }
      /* Fall */
    case 5:
    	if(Get_Left_Encoder_Count() < DISTANCE_CROSS_THIRD && 
         Get_Right_Encoder_Count() < DISTANCE_CROSS_THIRD) {
    	  pwm_left_drive = pwm_right_drive = 255 - MAGNITUDE;
        break;
      } else {
    	  pwm_left_drive = pwm_right_drive = 127;
        ++state;
        wait = 3;
      }
      /* Fall */
    case 6:
      if (--wait != 0) {
        break;
      } else {
        Reset_Left_Encoder_Count();
        Reset_Right_Encoder_Count();        
        state = 3;
        goto statethree;
      }
  }

}

const int CORNER_DUMP_DISTANCE = 
    DISTANCE_LEN_HALF - DISTANCE_CROSS_1SPOS;
void Corner_Dump(void)
{
  switch (state) {
    case 0:
        Reset_Left_Encoder_Count();
        Reset_Right_Encoder_Count();
        ++state;
        /* Fall through */
    case 1:
      // Drive forward
    	if(Get_Left_Encoder_Count() < CORNER_DUMP_DISTANCE && 
         Get_Right_Encoder_Count() < CORNER_DUMP_DISTANCE) {
    	  pwm_left_drive = pwm_right_drive = 255 - MAGNITUDE;
        break;
      } else {
    	  pwm_left_drive = pwm_right_drive = 127;
        ++state;
      }
      // Fall
    case 2:
      // Dump balls
      pwm_collector_convey = 255;
      // Continue to drive at a nominal rate to
      // maintain position on ramp
      pwm_left_drive = pwm_right_drive = 255 - MAGNITUDE;
      break;
  }
}
